@@ -4,6 +4,7 @@ import itertools
 import re
 import warnings
 from collections import ChainMap
+from datetime import datetime
 from typing import (
     Any,
     Callable,
@@ -24,9 +25,17 @@ import xarray as xr
 from xarray import DataArray, Dataset
 from xarray.core.arithmetic import SupportsArithmetic
 
-from .criteria import coordinate_criteria, regex
+from .criteria import cf_role_criteria, coordinate_criteria, regex
 from .helpers import bounds_to_vertices
-from .utils import _is_datetime_like, invert_mappings, parse_cell_methods_attr
+from .options import OPTIONS
+from .utils import (
+    _get_version,
+    _is_datetime_like,
+    always_iterable,
+    invert_mappings,
+    parse_cell_methods_attr,
+    parse_cf_standard_name_table,
+)
 
 #: Classes wrapped by cf_xarray.
 _WRAPPED_CLASSES = (
@@ -57,7 +66,6 @@ ATTRS = {
 ATTRS["time"] = ATTRS["T"]
 ATTRS["vertical"] = ATTRS["Z"]
 
-
 # Type for Mapper functions
 Mapper = Callable[[Union[DataArray, Dataset], str], List[str]]
 
@@ -68,7 +76,7 @@ F = TypeVar("F", bound=Callable[..., Any])
 def apply_mapper(
     mappers: Union[Mapper, Tuple[Mapper, ...]],
     obj: Union[DataArray, Dataset],
-    key: str,
+    key: Any,
     error: bool = True,
     default: Any = None,
 ) -> List[Any]:
@@ -79,8 +87,13 @@ def apply_mapper(
     It should return a list in all other cases including when there are no
     results for a good key.
     """
-    if default is None:
-        default = []
+
+    if not isinstance(key, str):
+        if default is None:
+            raise ValueError("`default` must be provided when `key` is not a string.")
+        return list(always_iterable(default))
+
+    default = [] if default is None else list(always_iterable(default))
 
     def _apply_single_mapper(mapper):
 
@@ -158,6 +171,55 @@ def _get_groupby_time_accessor(var: Union[DataArray, Dataset], key: str) -> List
         return []
 
 
+def _get_custom_criteria(
+    obj: Union[DataArray, Dataset], key: str, criteria=None
+) -> List[str]:
+    """
+    Translate from axis, coord, or custom name to variable name optionally
+    using ``custom_criteria``
+
+    Parameters
+    ----------
+    obj : DataArray, Dataset
+    key : str
+        key to check for.
+    criteria : dict, optional
+        Criteria to use to map from variable to attributes describing the
+        variable. An example is coordinate_criteria which maps coordinates to
+        their attributes and attribute values. If user has defined
+        custom_criteria, this will be used by default.
+
+    Returns
+    -------
+    List[str], Variable name(s) in parent xarray object that matches axis, coordinate, or custom `key`
+
+    """
+
+    if isinstance(obj, DataArray):
+        obj = obj._to_temp_dataset()
+
+    if criteria is None:
+        if not OPTIONS["custom_criteria"]:
+            return []
+        criteria = OPTIONS["custom_criteria"]
+
+    if criteria is not None:
+        criteria = always_iterable(criteria, allowed=(tuple, list, set))
+
+    criteria = ChainMap(*criteria)
+
+    results: Set = set()
+    if key in criteria:
+        for criterion, patterns in criteria[key].items():
+            for var in obj.variables:
+                if re.match(patterns, obj[var].attrs.get(criterion, "")):
+                    results.update((var,))
+                # also check name specifically since not in attributes
+                elif criterion == "name" and re.match(patterns, var):
+                    results.update((var,))
+    return list(results)
+
+
 def _get_axis_coord(var: Union[DataArray, Dataset], key: str) -> List[str]:
     """
     Translate from axis or coord name to variable name
@@ -210,15 +272,13 @@ def _get_axis_coord(var: Union[DataArray, Dataset], key: str) -> List[str]:
 
     results: Set = set()
     for coord in search_in:
-        for criterion, valid_values in coordinate_criteria.items():
-            if key in valid_values:
-                expected = valid_values[key]
+        if key in coordinate_criteria:
+            for criterion, expected in coordinate_criteria[key].items():
                 if (
                     coord in var.coords
                     and var.coords[coord].attrs.get(criterion, None) in expected
                 ):
                     results.update((coord,))
-
     return list(results)
 
 
@@ -304,7 +364,13 @@ def _get_all(obj: Union[DataArray, Dataset], key: str) -> List[str]:
     One or more of ('X', 'Y', 'Z', 'T', 'longitude', 'latitude', 'vertical', 'time',
     'area', 'volume'), or arbitrary measures, or standard names
     """
-    all_mappers = (_get_axis_coord, _get_measure, _get_with_standard_name)
+    all_mappers = (
+        _get_custom_criteria,
+        functools.partial(_get_custom_criteria, criteria=cf_role_criteria),
+        _get_axis_coord,
+        _get_measure,
+        _get_with_standard_name,
+    )
     results = apply_mapper(all_mappers, obj, key, error=False, default=None)
     return results
 
@@ -576,6 +642,8 @@ def _getitem(
         measures = []
         warnings.warn("Ignoring bad cell_measures attribute.", UserWarning)
 
+    custom_criteria = ChainMap(*OPTIONS["custom_criteria"])
+
     varnames: List[Hashable] = []
     coords: List[Hashable] = []
     successful = dict.fromkeys(key, False)
@@ -592,6 +660,11 @@ def _getitem(
             successful[k] = bool(measure)
             if measure:
                 varnames.extend(measure)
+        elif k in custom_criteria or k in cf_role_criteria:
+            names = _get_all(obj, k)
+            check_results(names, k)
+            successful[k] = bool(names)
+            varnames.extend(names)
         else:
             stdnames = set(_get_with_standard_name(obj, k))
             objcoords = set(obj.coords)
@@ -917,8 +990,7 @@ class CFAccessor:
             value = kwargs[key]
             mappers = all_mappers[key]
 
-            if isinstance(value, str):
-                value = [value]
+            value = always_iterable(value)
 
             if isinstance(value, dict):
                 # this for things like isel where **kwargs captures things like T=5
@@ -1317,7 +1389,7 @@ class CFAccessor:
             Variables will be renamed to match variable names in this xarray object
         skip: str, Iterable[str], optional
             Limit the renaming excluding
-            ("axes", "cell_measures", "coordinates", "standard_names")
+            ("axes", "bounds", cell_measures", "coordinates", "standard_names")
             or a subset thereof.
 
         Returns
@@ -1332,24 +1404,48 @@ class CFAccessor:
         good_keys = ourkeys & theirkeys
         keydict = {}
         for key in good_keys:
-            ours = set(_get_all(self._obj, key))
-            theirs = set(_get_all(other, key))
+            ours = set(apply_mapper(_get_all, self._obj, key))
+            theirs = set(apply_mapper(_get_all, other, key))
             for attr in skip:
-                ours -= set(getattr(self, attr).get(key, []))
-                theirs -= set(getattr(other.cf, attr).get(key, []))
+                ours.difference_update(getattr(self, attr).get(key, []))
+                theirs.difference_update(getattr(other.cf, attr).get(key, []))
             if ours and theirs:
                 keydict[key] = dict(ours=list(ours), theirs=list(theirs))
 
-        conflicts = {}
-        for k0, v0 in keydict.items():
-            if len(v0["ours"]) > 1 or len(v0["theirs"]) > 1:
-                conflicts[k0] = v0
-                continue
-            for v1 in keydict.values():
-                # Conflicts have same ours but different theirs or vice versa
-                if (v0["ours"] == v1["ours"]) != (v0["theirs"] == v1["theirs"]):
+        def get_renamer_and_conflicts(keydict):
+            conflicts = {}
+            for k0, v0 in keydict.items():
+                if len(v0["ours"]) > 1 or len(v0["theirs"]) > 1:
                     conflicts[k0] = v0
-                    break
+                    continue
+                for v1 in keydict.values():
+                    # Conflicts have same ours but different theirs or vice versa
+                    if (v0["ours"] == v1["ours"]) != (v0["theirs"] == v1["theirs"]):
+                        conflicts[k0] = v0
+                        break
+
+            renamer = {
+                v["ours"][0]: v["theirs"][0]
+                for k, v in keydict.items()
+                if k not in conflicts
+            }
+
+            return renamer, conflicts
+
+        # Run get_renamer_and_conflicts twice.
+        # The second time add the bounds associated with variables to rename
+        renamer, conflicts = get_renamer_and_conflicts(keydict)
+        if "bounds" not in skip:
+            for k, v in renamer.items():
+                ours = set(getattr(self, "bounds", {}).get(k, []))
+                theirs = set(getattr(other.cf, "bounds", {}).get(v, []))
+                if ours and theirs:
+                    ours.update(keydict.get(k, {}).get("ours", []))
+                    theirs.update(keydict.get(k, {}).get("theirs", []))
+                    keydict[k] = dict(ours=list(ours), theirs=list(theirs))
+            renamer, conflicts = get_renamer_and_conflicts(keydict)
+
+        # Rename and warn
         if conflicts:
             warnings.warn(
                 "Conflicting variables skipped:\n"
@@ -1363,23 +1459,28 @@ class CFAccessor:
                 ),
                 UserWarning,
             )
-
-        renamer = {
-            v["ours"][0]: v["theirs"][0]
-            for k, v in keydict.items()
-            if k not in conflicts
-        }
         newobj = self._obj.rename(renamer)
 
-        # rename variable names in the coordinates attribute
+        # rename variable names in the attributes
         # if present
         ds = self._maybe_to_dataset(newobj)
         for _, variable in ds.variables.items():
-            coordinates = variable.attrs.get("coordinates", None)
-            if coordinates:
-                for k, v in renamer.items():
-                    coordinates = coordinates.replace(k, v)
-                variable.attrs["coordinates"] = coordinates
+            for attr in ("bounds", "coordinates", "cell_measures"):
+                if attr == "cell_measures":
+                    varlist = [
+                        f"{k}: {renamer.get(v, v)}"
+                        for k, v in parse_cell_methods_attr(
+                            variable.attrs.get(attr, "")
+                        ).items()
+                    ]
+                else:
+                    varlist = [
+                        renamer.get(var, var)
+                        for var in variable.attrs.get(attr, "").split()
+                    ]
+
+                if varlist:
+                    variable.attrs[attr] = " ".join(varlist)
         return self._maybe_to_dataarray(ds)
 
     def guess_coord_axis(self, verbose: bool = False) -> Union[DataArray, Dataset]:
@@ -1485,6 +1586,110 @@ class CFAccessor:
                 result *= -1
         return result
 
+    def add_canonical_attributes(
+        self,
+        override: bool = False,
+        skip: Union[str, List[str]] = None,
+        verbose: bool = False,
+        source=None,
+    ) -> Union[Dataset, DataArray]:
+        """
+        Add canonical CF attributes to variables with standard names.
+        Attributes are parsed from the official CF standard name table [1]_.
+        This function adds an entry to the "history" attribute.
+
+        Parameters
+        ----------
+        override: bool
+            Override existing attributes.
+        skip: str, iterable, optional
+            Attribute(s) to skip: ``{"units", "grib", "amip", "description"}``.
+        verbose: bool
+            Print added attributes to screen.
+        source: optional
+            Path of `cf-standard-name-table.xml` or file object containing XML data.
+            If ``None``, use the default version associated with ``cf-xarray``.
+
+        Returns
+        -------
+        DataArray or Dataset with attributes added.
+
+        Notes
+        -----
+        The ``"units"`` attribute is never added to datetime-like variables.
+
+        References
+        ----------
+        .. [1] https://cfconventions.org/standard-names.html
+        """
+
+        # Arguments to add to history
+        args = ", ".join([f"{k!s}={v!r}" for k, v in locals().items() if k != "self"])
+
+        # Defaults
+        skip = [skip] if isinstance(skip, str) else (skip or [])
+
+        # Parse table
+        info, table, aliases = parse_cf_standard_name_table(source)
+
+        # Loop over standard names
+        ds = self._maybe_to_dataset().copy()
+        attrs_to_print: dict = {}
+        for std_name, var_names in ds.cf.standard_names.items():
+
+            # Loop over variable names
+            for var_name in var_names:
+                old_attrs = ds[var_name].attrs
+                std_name = aliases.get(std_name, std_name)
+                new_attrs = table.get(std_name, {})
+
+                # Loop over attributes
+                for key, value in new_attrs.items():
+                    if value and key not in skip and (override or key not in old_attrs):
+
+                        # Don't add units to time variables (e.g., datetime64, ...)
+                        if key == "units" and _is_datetime_like(ds[var_name]):
+                            continue
+
+                        # Add attribute
+                        ds[var_name].attrs[key] = value
+
+                        # Build verbose dictionary
+                        if verbose:
+                            attrs_to_print.setdefault(var_name, {})
+                            attrs_to_print[var_name][key] = value
+
+        if verbose:
+            # Info
+            strings = ["CF Standard Name Table info:"]
+            for key, value in info.items():
+                strings.append(f"- {key}: {value}")
+
+            # Attributes added
+            strings.append("\nAttributes added:")
+            for varname, attrs in attrs_to_print.items():
+                strings.append(f"- {varname}:")
+                for key, value in attrs.items():
+                    strings.append(f"    * {key}: {value}")
+                strings.append("")
+
+            print("\n".join(strings))
+
+        # Prepend history
+        now = datetime.now().ctime()
+        method_name = inspect.stack()[0][3]
+        version = _get_version()
+        table_version = info["version_number"]
+        history = (
+            f"{now}:"
+            f" cf.{method_name}({args})"
+            f" [cf-xarray {version}, cf-standard-name-table {table_version}]\n"
+        )
+        obj = self._maybe_to_dataarray(ds)
+        obj.attrs["history"] = history + obj.attrs.get("history", "")
+
+        return obj
+
 
 @xr.register_dataset_accessor("cf")
 class CFDatasetAccessor(CFAccessor):
