@@ -1,5 +1,6 @@
 import itertools
 from textwrap import dedent
+from urllib.request import urlopen
 
 import matplotlib as mpl
 import numpy as np
@@ -11,6 +12,7 @@ from xarray import Dataset
 from xarray.testing import assert_allclose, assert_identical
 
 import cf_xarray  # noqa
+from cf_xarray.utils import parse_cf_standard_name_table
 
 from ..datasets import (
     airds,
@@ -241,7 +243,7 @@ def test_rename_like():
     for k in ["TLONG", "TLAT"]:
         assert k not in renamed.coords
         assert k in original.coords
-        assert original.TEMP.attrs["coordinates"] == "TLONG TLAT"
+    assert original.TEMP.attrs["coordinates"] == "TLONG TLAT"
 
     assert "lon" in renamed.coords
     assert "lat" in renamed.coords
@@ -271,6 +273,20 @@ def test_rename_like():
     actual = da.cf.rename_like(airds, skip="axes").cf.coordinates
     assert expected == actual
 
+    # rename bounds
+    original = airds.cf[["air"]].cf.add_bounds("lon")
+    other = popds.cf[["TEMP"]].cf.add_bounds("nlon")
+    renamed = original.cf.rename_like(other, skip="coordinates")
+    assert renamed.cf.bounds["nlon"] == ["nlon_bounds"]
+
+    # rename cell measures
+    other = airds.cf["air"].cf.rename(area="CELL_AREA")
+    other.attrs["cell_measures"] = other.attrs["cell_measures"].replace(
+        "cell_area", "CELL_AREA"
+    )
+    renamed = airds.cf["air"].cf.rename_like(other)
+    assert renamed.cf.cell_measures["area"] == ["CELL_AREA"]
+
 
 @pytest.mark.parametrize("obj", objects)
 @pytest.mark.parametrize(
@@ -317,8 +333,13 @@ def test_weighted(obj):
     with raise_if_dask_computes(max_computes=2):
         # weights are checked for nans
         expected = obj.weighted(obj["cell_area"]).sum("lat")
-        actual = obj.cf.weighted("area").sum("Y")
-    assert_identical(expected, actual)
+        actuals = [
+            obj.cf.weighted("area").sum("Y"),
+            obj.cf.weighted(obj["cell_area"]).sum("Y"),
+            obj.cf.weighted(weights=obj["cell_area"]).sum("Y"),
+        ]
+    for actual in actuals:
+        assert_identical(expected, actual)
 
 
 @pytest.mark.parametrize("obj", objects)
@@ -398,7 +419,7 @@ def test_args_methods(obj):
 
 def test_dataarray_getitem():
 
-    air = airds.air
+    air = airds.air.copy()
     air.name = None
 
     assert_identical(air.cf["longitude"], air["lon"])
@@ -1214,3 +1235,144 @@ def test_cmip6_attrs():
     )
     assert da.cf.axes["X"] == ["nlon"]
     assert da.cf.axes["Y"] == ["nlat"]
+
+
+def test_custom_criteria():
+    my_custom_criteria = {
+        "ssh": {
+            "standard_name": "sea_surface_elev*|sea_surface_height",
+            "name": "sea_surface_elevation$",  # variable name
+        },
+        "salt": {
+            "standard_name": "salinity",
+            "name": "sal*",
+        },
+        "wind_speed": {
+            "standard_name": "wind_speed$",
+        },
+    }
+    my_custom_criteria2 = {"temp": {"name": "temperature"}}
+    my_custom_criteria_list = [my_custom_criteria, my_custom_criteria2]
+    my_custom_criteria_tuple = (my_custom_criteria, my_custom_criteria2)
+
+    cf_xarray.set_options(custom_criteria=my_custom_criteria)
+
+    # Match by name regex match
+    ds = xr.Dataset()
+    ds["salinity"] = ("dim", np.arange(10))
+    assert_identical(ds.cf["salt"], ds["salinity"])
+
+    # Match by standard_name regex match
+    ds = xr.Dataset()
+    ds["elev"] = ("dim", np.arange(10), {"standard_name": "sea_surface_elevBLAH"})
+    assert_identical(ds.cf["ssh"], ds["elev"])
+
+    # Match by standard_name exact match
+    ds = xr.Dataset()
+    ds["salinity"] = ("dim", np.arange(10), {"standard_name": "salinity"})
+    assert_identical(ds.cf["salt"], ds["salinity"])
+
+    # If not exact name, won't match
+    ds = xr.Dataset()
+    ds["sea_surface_elevation123"] = ("dim", np.arange(10))
+    # Since this will not match, this should error
+    with pytest.raises(KeyError):
+        ds.cf["ssh"]
+
+    # will select only one variable here since exact match
+    ds = xr.Dataset()
+    ds["winds"] = ("dim", np.arange(10), {"standard_name": "wind_speed"})
+    ds["gusts"] = ("dim", np.arange(10), {"standard_name": "wind_speed_of_gust"})
+    assert_identical(ds.cf["wind_speed"], ds["winds"])
+
+    # Match by exact name
+    ds = xr.Dataset()
+    ds["sea_surface_elevation"] = ("dim", np.arange(10))
+    ds["sea_surface_height"] = (
+        "dim",
+        np.arange(10),
+        {"standard_name": "sea_surface_elevBLAH"},
+    )
+    # Since there are two variables, this should error
+    with pytest.raises(KeyError):
+        ds.cf["ssh"]
+    # But the following should work instead given the two ssh variables
+    assert_identical(
+        ds.cf[["ssh"]], ds[["sea_surface_elevation", "sea_surface_height"]]
+    )
+
+    # test criteria list of dicts
+    with cf_xarray.set_options(custom_criteria=my_custom_criteria_list):
+        ds = xr.Dataset()
+        ds["temperature"] = ("dim", np.arange(10))
+        assert_identical(ds.cf["temp"], ds["temperature"])
+
+    # test criteria tuple of dicts
+    with cf_xarray.set_options(custom_criteria=my_custom_criteria_tuple):
+        ds = xr.Dataset()
+        ds["temperature"] = ("dim", np.arange(10))
+        assert_identical(ds.cf["temp"], ds["temperature"])
+
+
+def test_cf_standard_name_table_version():
+
+    url = (
+        "https://raw.githubusercontent.com/cf-convention/cf-convention.github.io/"
+        "master/Data/cf-standard-names/current/src/cf-standard-name-table.xml"
+    )
+    expected_info, _, _ = parse_cf_standard_name_table(urlopen(url))
+    actual_info, _, _ = parse_cf_standard_name_table()
+    assert expected_info == actual_info
+
+
+@pytest.mark.parametrize("override", [True, False])
+@pytest.mark.parametrize("skip", ["units", None])
+@pytest.mark.parametrize("verbose", [True, False])
+def test_add_canonical_attributes(override, skip, verbose, capsys):
+
+    ds = airds
+    cf_ds = ds.cf.add_canonical_attributes(
+        override=override, skip=skip, verbose=verbose
+    )
+
+    # Catch print
+    captured = capsys.readouterr()
+    if not verbose:
+        captured.out == ""
+
+    # Attributes have been added
+    for var in sum(ds.cf.standard_names.values(), []):
+        assert set(ds[var].attrs) < set(cf_ds[var].attrs)
+
+    # Time units did not change
+    assert ds["time"].attrs.get("units") is cf_ds["time"].attrs.get("units") is None
+
+    # Check override, skip, and verbose
+    if not override or skip:
+        assert cf_ds["lat"].attrs["units"] == "degrees_north"
+        assert "* units" not in captured.out
+    else:
+        assert cf_ds["lat"].attrs["units"] == "degree_north"
+        if verbose:
+            assert "* units: degree_north" in captured.out
+
+    # History
+    assert (
+        f"cf.add_canonical_attributes(override={override!r}, skip={skip!r}, verbose={verbose!r}, source=None)"
+        in cf_ds.attrs["history"]
+    )
+
+    # DataArray (test only once)
+    if override and skip and verbose:
+        cf_da = ds["air"].cf.add_canonical_attributes(
+            override=override, skip=skip, verbose=verbose
+        )
+
+        time_stamp_size = 24
+        assert (
+            cf_da.attrs["history"][time_stamp_size:]
+            == cf_ds.attrs["history"][time_stamp_size:]
+        )
+
+        cf_da.attrs.pop("history")
+        assert_identical(cf_da, cf_ds["air"])
